#!/bin/sh

# Copyright (C) 2019-2023 Svante Signell

# Written by Svante Signell  <svante.signell@gmail.com>

# This file is part of hurd-cross.

# Hurd-cross is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 3 or (at your
# option) any later version.

# Hurd-cross is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with hurd-cross.  If not, see <http://www.gnu.org/licenses/>.

. ./package-versions.sh
. ./environment-variables.sh

echo ""
echo "File: bootstrap-funcs.sh"

# You can change the GCC version here.
#export CC="$HOST_MACHINE"-gcc
#export CXX="$HOST_MACHINE"-g++
#export AR="$HOST_MACHINE-ar"
#export AS="$HOST_MACHINE-as"

compile_binutils ()
{
   print_info "Cross-compiling BINUTILS"
   rm -rf "$BINUTILS_SRC".obj
   mkdir -p "$BINUTILS_SRC".obj
   cd "$BINUTILS_SRC".obj
   ../$BINUTILS_SRC/configure \
     --build="$HOST" \
     --host="$HOST" \
     --target="$TARGET" \
     --prefix="$CROSS_ROOT" \
     --enable-static \
     --disable-multilib \
     --disable-werror \
     --disable-nls \
     --with-system-zlib
   make -j$PROCS all
   fakeroot make install
   cd ..
   rm -rf "$BINUTILS_SRC".obj
}

install_gnumach_headers()
{
   print_info "Installing GNUMACH Headers"
   cd "$GNUMACH_SRC"
   autoreconf -i
   cd ..
   rm -rf "$GNUMACH_SRC".obj
   mkdir -p "$GNUMACH_SRC".obj
   cd "$GNUMACH_SRC".obj
   ../$GNUMACH_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$CROSS_ROOT"
   make -j$PROCS install-data
   cd -
   rm -rf "$GNUMACH_SRC".obj
}

install_hurd_headers()
{
   print_info "Installing HURD headers"
   apply_patch $SCRIPT_DIR/patches/hurd/configure.ac.diff 1
   cd "$HURD_SRC"
   autoreconf -i
   cd ..
   rm -rf "$HURD_SRC".obj
   mkdir -p "$HURD_SRC".obj
   cd "$HURD_SRC".obj
   ../$HURD_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$CROSS_ROOT" \
     --disable-profile \
     --without-parted
   fakeroot make -j$PROCS install-headers no_deps=t
   cd ..
   unapply_patch $SCRIPT_DIR/patches/hurd/configure.ac.diff 1
   rm -rf "$HURD_SRC".obj
}

compile_first_gcc ()
{
   print_info "Cross-compiling first stage of GCC"
   #Note: This does not work unless autoconf2.69 is installed.
   #cd "$GCC_SRC"
   #autoreconf -i
   #cd ..
   rm -rf "$GCC_SRC".obj
   mkdir -p "$GCC_SRC".obj
   cd "$GCC_SRC".obj
   CPPLAGS="-ffreestanding" \
   LDFLAGS="-Wl,-rpath,${CROSS_ROOT}/lib" \
   ../$GCC_SRC/configure \
     --build="$HOST" \
     --host="$HOST" \
     --target="$TARGET" \
     --prefix="$CROSS_ROOT" \
     --enable-languages=c \
     --enable-threads=posix \
     --disable-bootstrap \
     --disable-shared \
     --disable-threads \
     --disable-libatomic \
     --enable-decimal-float \
     --disable-libffi \
     --disable-libgomp \
     --disable-libitm \
     --disable-libmpx \
     --disable-libquadmath \
     --without-headers \
     --with-newlib \
     --disable-nls \
     --disable-multilib \
     --disable-target-zlib \
     --with-system-zlib \
     --disable-libssp \
     --disable-libvtv \
     --disable-libstdcxx
   make -j$PROCS all-gcc
   fakeroot make install-gcc
   make -j$PROCS configure-target-libgcc
   cd "$TARGET"/libgcc
   make -j$PROCS all
   fakeroot make install
   cd ../..
#   cd "$GCC_SRC"
   cd ..
   rm -rf "$GCC_SRC".obj
}

# check if needed:
#   TARGET_CC="$TARGET-gcc" \

install_mig() {
   print_info "Cross-compiling MIG"
   cd "$MIG_SRC"
   autoreconf -i
   cd ..
   rm -rf "$MIG_SRC".obj
   mkdir -p "$MIG_SRC".obj
   cd "$MIG_SRC".obj
   TARGET_CPPFLAGS="-I$CROSS_ROOT/include" \
   ../$MIG_SRC/configure \
     --build="$HOST" \
     --target="$TARGET" \
     --prefix="$CROSS_ROOT"
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$MIG_SRC".obj
}

#Use the following patch: git-mach-headers-freestanding.diff
compile_first_glibc() {
   print_info "Cross-compiling first stage of GLIBC"
#   cd "$GLIBC_SRC"
#   autoreconf -i
#   cd ..
   rm -rf "$GLIBC_SRC".obj
   mkdir -p "$GLIBC_SRC".obj
   cd "$GLIBC_SRC".obj
   ../$GLIBC_SRC/configure \
      --with-binutils=$CROSS_ROOT/bin \
      --build="$HOST" \
      --host="$TARGET" \
      --prefix="$CROSS_ROOT" \
      --with-headers="$CROSS_ROOT"/include \
      --cache-file=config.cache \
      --enable-shared \
      --disable-static \
      --disable-mathvec \
      --disable-profile \
      --enable-add-ons=libpthread \
      --enable-obsolete-rpc \
      --disable-werror \
      --disable-nscd
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$GLIBC_SRC".obj
}

#FIXME: fixinc_glimits.h.diff still needed?
# Check if the CPPFLAG below is sufficient:
# CPPFLAGS="-I$TARGET_ROOT/include -ffreestanding"
compile_full_gcc () {
   print_info "Cross-compiling full GCC"
   cd "$GCC_SRC"
   apply_patch $SCRIPT_DIR/patches/gcc/fixinc_glimits.h.diff 1
   #Note: This does not work unless autoconf2.69 is installed.
   #autoreconf -i
   cd ..
   rm -rf "$GCC_SRC".obj
   mkdir -p "$GCC_SRC".obj
   cd "$GCC_SRC".obj 
   CPPLAGS="-ffreestanding" \
   ../$GCC_SRC/configure \
     --target="$TARGET" \
     --prefix="$CROSS_ROOT" \
     --disable-static \
     --disable-nls \
     --enable-languages=c,c++ \
     --enable-threads=posix \
     --disable-multilib \
     --with-system-zlib \
     --with-libstdcxx-time \
     --disable-libstdcxx-pch \
     --disable-bootstrap \
     --disable-libcilkrts \
     --disable-libgomp \
     --disable-multi-arch \
     --with-arch=$CPU
   make -j$PROCS all
   fakeroot make install
   cd ..
   cd $GCC_SRC
   unapply_patch $SCRIPT_DIR/patches/gcc/fixinc_glimits.h.diff 1
   cd ..
#   rm -rf "$GCC_SRC".obj
}

#FIXED: Add --enable-add-ons=libpthread
compile_full_glibc() {
   print_info "Cross-compiling full GLIBC"
   rm -rf "$GLIBC_SRC".obj
   mkdir -p "$GLIBC_SRC".obj
   cd "$GLIBC_SRC".obj
   rm -f config.cache
   CPPFLAGS="-fno-omit-frame-pointer" \
   ../$GLIBC_SRC/configure \
      --with-binutils=$CROSS_ROOT/bin \
      --build="$HOST" \
      --host="$TARGET" \
      --prefix="$CROSS_ROOT" \
      --with-headers="$CROSS_ROOT"/include \
      --enable-obsolete-rpc \
      --disable-profile \
      --enable-add-ons=libpthread \
      --enable-obsolete-rpc \
      --disable-werror \
      --disable-nscd
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$GLIBC_SRC".obj
}
