#!/bin/sh

# Copyright (C) 2019-2023 Svante Signell

# Written by Svante Signell  <svante.signell@gmail.com>

# This file is part of hurd-cross.

# Hurd-cross is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 3 or (at your
# option) any later version.

# Hurd-cross is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with hurd-cross.  If not, see <http://www.gnu.org/licenses/>.

. ./package-versions.sh
. ./environment-variables.sh

echo ""
echo "File: compile-funcs.sh"

# Show the actual compiler and linker commands it is executing. 
# For CMake, use make VERBOSE=1; for GNU autotools make V=1

export CONFIG_SYSTEM=$SYSTEM/../config-site
export CC="${CROSS_ROOT}/bin/${TARGET}-gcc"
export CXX="${CROSS_ROOT}/bin/${TARGET}-g++"
export AR="${CROSS_ROOT}/bin/${TARGET}-ar"
export AS="${CROSS_ROOT}/bin/${TARGET}-as"
export RANLIB="${CROSS_ROOT}/bin/${TARGET}-ranlib"
export LD="${CROSS_ROOT}/bin/${TARGET}-ld"
export STRIP="${CROSS_ROOT}/bin/${TARGET}-strip"
export MIG="${CROSS_ROOT}/bin/${TARGET}-mig"
export PKG_CONFIG=/usr/bin/pkg-config
export PKG_CONFIG_LIBDIR="$TARGET_ROOT"/lib/pkgconfig

install_zlib()
{
   print_info "Installing zlib"
   rm -rf "$ZLIB_SRC".obj
   mkdir -p "$ZLIB_SRC".obj
   cd "$ZLIB_SRC".obj
   CHOST=i686-pc-gnu \
   ../$ZLIB_SRC/configure \
     --prefix=$TARGET_ROOT
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$ZLIB_SRC".obj
}

#FIXME: Check if config-site/flex-site.hurd-i386 is needed
install_flex()
{
   print_info "Installing flex"
   cd "$FLEX_SRC"
   autoreconf -i
   cd ..
   rm -rf "$FLEX_SRC".obj
   mkdir -p "$FLEX_SRC".obj
   cd "$FLEX_SRC".obj
   CONFIG_SITE=$CONFIG_SYSTEM/flex-site.hurd-i386 \
     ../$FLEX_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT"
   sed -i -e 's/tests//' Makefile
   make -j$PROCS all
   fakeroot make install
   cd ..
   rm -rf "$FLEX_SRC".obj
}

install_mig() {
   print_info "Installing mig"
   cd "$MIG_SRC"
   autoreconf -i
   cd ..
   rm -rf "$MIG_SRC".obj
   mkdir -p "$MIG_SRC".obj
   cd "$MIG_SRC".obj
   rm -f config.cache
   LDFLAGS="-L$TARGET_ROOT/lib" \
   ../$MIG_SRC/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --target="$TARGET" \
      --prefix="$TARGET_ROOT"
   make -j$PROCS all
   fakeroot make install
   cd ..
   rm -rf "$MIG_SRC".obj
}

install_gnumach()
{
   print_info "Installing gnumach"
   cd "$GNUMACH_SRC"
   autoreconf -i
   cd ..
   rm -rf "$GNUMACH_SRC".obj
   mkdir -p "$GNUMACH_SRC".obj
   LDFLAGS="-L$TARGET_ROOT/lib" \
   cd "$GNUMACH_SRC".obj
   ../$GNUMACH_SRC/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --prefix="$TARGET_ROOT" \
      --enable-kdb \
      --enable-kmsg
   make -j$PROCS gnumach.gz gnumach gnumach.msgids
   fakeroot make install
   mkdir -p $TARGET_ROOT/boot
   cp gnumach.gz $TARGET_ROOT/boot
   cd -
   rm -rf "$GNUMACH_SRC".obj
}

install_pkg-config()
{
   print_info "Installing pkg-config"
   cd "$PKG_CONFIG_SRC"
   # Needed for pkg-config/glib to build!!
   autoreconf -i
   cd ..
   rm -rf "$PKG_CONFIG_SRC".obj
   mkdir -p "$PKG_CONFIG_SRC".obj
   cd "$PKG_CONFIG_SRC".obj
   CONFIG_SITE=$CONFIG_SYSTEM/pkg-config-site.hurd-i386 \
     ../"$PKG_CONFIG_SRC"/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --disable-shared \
     --with-internal-glib
   make -j$PROCS all-recursive
   fakeroot make install
   cd ..
   rm -rf "$PKG_CONFIG_SRC".obj
}

install_binutils ()
{
   print_info "Installing binutils"
   cd "$BINUTILS_SRC"
   autoreconf -i
   cd ..
   rm -rf "$BINUTILS_SRC".obj
   mkdir -p "$BINUTILS_SRC".obj
   cd "$BINUTILS_SRC".obj
   ../$BINUTILS_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --target="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --disable-nls \
     --enable-shared \
     --disable-multilib
   make -j$PROCS all
   fakeroot make install
   cd ..
   rm -rf "$BINUTILS_SRC".obj
}

install_sed()
{
   print_info "Installing sed"
   cd "$SED_SRC"
   autoreconf -i
   cd ..
   rm -rf "$SED_SRC".obj
   mkdir -p "$SED_SRC".obj
   cd "$SED_SRC".obj
   ../$SED_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT"
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$SED_SRC".obj
}

install_coreutils()
{
   print_info "Installing coreutils"
   cd "$COREUTILS_SRC"
   autoreconf -i
   cd ..
   rm -rf "$COREUTILS_SRC".obj
   mkdir -p "$COREUTILS_SRC".obj
   cd "$COREUTILS_SRC".obj
   ../$COREUTILS_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --enable-install-program=hostname \
     --cache-file=config.cache
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$COREUTILS_SRC".obj
}

#FIXME: Needs gettext, gperf, texinfo: OK w/o
#FIXME: Reentrant code: define NCURSES_REENTRANT
#    --with-pthread \
#    --enable-reentrant
install_ncurses ()
{
  print_info "Installing ncurses"
# Don't: Requires autoconf-dickey
#  cd "$NCURSES_SRC"
#   autoreconf -i
#   cd ..
#
  rm -rf "$NCURSES_SRC".obj
  mkdir -p "$NCURSES_SRC".obj
  cd "$NCURSES_SRC".obj
  ../$NCURSES_SRC/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --prefix="$TARGET_ROOT" \
      --includedir=$TARGET_ROOT/include/ncursesw \
      --with-shared \
      --without-debug \
      --without-ada \
      --enable-pc-files \
      --with-ticlib=tic \
      --with-termlib=tinfo \
      --disable-termcap \
      --enable-symlinks \
      --enable-overwrite \
      --with-build-cc=gcc
 make -j$PROCS
 fakeroot make install
 # wide-char/UTF-8 libraries are needed for util-linux (and hurd??)
 # libtinfow* in not built??
  ../$NCURSES_SRC/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --prefix="$TARGET_ROOT" \
      --with-shared \
      --with-ncursesw \
      --with-termlib=tinfow \
      --with-progs \
      --enable-widec
  make -j$PROCS libs
  fakeroot make install.libs
  cd ..
  rm -rf "$NCURSES_SRC".obj
}

install_pam ()
{
  print_info "Installing pam"
  cd "$PAM_SRC"
  autoreconf -i
  cd ..
  rm -rf "$PAM_SRC".obj
  mkdir -p "$PAM_SRC".obj
  cd "$PAM_SRC".obj
  ../$PAM_SRC/configure \
    --build="$HOST" \
    --host="$TARGET" \
    --prefix="$TARGET_ROOT" \
    --includedir="$TARGET_ROOT"/include/security \
    --disable-nis \
    --enable-db=no
  make -j$PROCS
  fakeroot make install
  cd ..
  rm -rf "$PAM_SRC".obj
}

install_bash()
{
   print_info "Installing bash"
   cd "$BASH_SRC"
   autoreconf -i
   cd ..
   rm -rf "$BASH_SRC".obj
   mkdir -p "$BASH_SRC".obj
   cd "$BASH_SRC".obj
   LDFLAGS="-L$TARGET_ROOT/lib -lncursesw -ltinfow" \
   ../$BASH_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --with-curses \
     --enable-readline \
     --without-bash-malloc
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$BASH_SRC".obj
}

install_libgpg-error()
{
   print_info "Installing libgpg-error"
   cd "$LIBGPG_ERROR_SRC"
   ./autogen.sh
   cd ..
   rm -rf "$LIBGPG_ERROR_SRC".obj
   mkdir -p "$LIBGPG_ERROR_SRC".obj
   cd "$LIBGPG_ERROR_SRC".obj
   ../$LIBGPG_ERROR_SRC/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --prefix="$TARGET_ROOT" \
      --enable-install-gpg-error-config
   make -j$PROCS all
   fakeroot make install
   cd ..
   rm -rf "$LIBGPG_ERROR_SRC".obj
}

install_libgcrypt()
{
   print_info "Installing libgcrypt"
   cd "$LIBGCRYPT_SRC"
   ./autogen.sh
   cd ..
   rm -rf "$LIBGCRYPT_SRC".obj
   mkdir -p "$LIBGCRYPT_SRC".obj
   cd "$LIBGCRYPT_SRC".obj
   LDFLAGS="-L$TARGET_ROOT/lib -lgpg-error" \
   ../$LIBGCRYPT_SRC/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --prefix="$TARGET_ROOT" \
      --with-libgpg-error-prefix="$TARGET_ROOT" \
      --enable-ld-version-script \
      --enable-noexecstack \
      --enable-shared \
      --enable-static
      make -j$PROCS all
      fakeroot make install
   cd ..
   rm -rf "$LIBGCRYPT_SRC".obj
}

#FIXME: lwip uses cmake, not built!!
#FIXME: Download liblwip0 instead!!
install_lwip() {
   print_info "Installing lwip"
   cd "$LWIP_SRC"
   autoreconf -i
   cd ..
   mkdir -p "$LWIP_SRC".obj
   cd "$LWIP_SRC".obj
   rm -f config.cache config.status
   ../$LWIP_SRC/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --prefix="$TARGET_ROOT" \
      --without-parted \
      --disable-profile
   make -j$PROCS all
   fakeroot make install
   cd ..
   rm -rf "$LWIP_SRC".obj
}

#Note: see logfile, /tools/lib/ld.so.conf needed?
install_libpciaccess()
{
   print_info "Installing libpciaccess"
   cd "$LIBPCIACCESS_SRC"
   autoreconf -i
   cd ..
   rm -rf "$LIBPCIACCESS_SRC".obj
   mkdir -p "$LIBPCIACCESS_SRC".obj
   cd "$LIBPCIACCESS_SRC".obj
   ../"$LIBPCIACCESS_SRC"/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --prefix="$TARGET_ROOT" \
      --enable-static
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$LIBPCIACCESS_SRC".obj
}

# Bug??: NCURSES_LIBS="-lncursesw -ltinfo" is not possible to override.
# Both libncurses/libtinfo and libncursesw/libtinfow are needed here!
install_util-linux()
{
   print_info "Installing util-linux"
   cd "$UTIL_LINUX_SRC"
   ./autogen.sh
   cd ..
   rm -rf "$UTIL_LINUX_SRC".obj
   mkdir -p "$UTIL_LINUX_SRC".obj
   cd "$UTIL_LINUX_SRC".obj
   CPPFLAGS="-I$TARGET_ROOT/include" \
   LDFLAGS="-Wl,-rpath-link,$SRC/$UTIL_LINUX_SRC.obj/.libs -L$TARGET_ROOT/lib -lncursesw -ltinfo -ltinfow" \
   ../$UTIL_LINUX_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --with-bashcompletiondir=$TARGET_ROOT/share/bash-completion/completions \
     --with-ncursesw \
     --enable-widechar \
     --without-python \
     --without-systemd \
     --without-udev \
     --without-audit \
     --enable-libmount-support-mtab \
     --disable-hardlink \
     --disable-login \
     --disable-nologin \
     --disable-kill \
     --disable-eject \
     --disable-chfn-chsh \
     --disable-ul \
     --disable-cal
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$UTIL_LINUX_SRC".obj
}

#NOTE: config-site/e2fsprogs-site.hurd-i386 is needed!
install_e2fsprogs()
{
    print_info "Installing e2fsprogs"
#   FIXME: autoreconf fails:
#   autoreconf: configure.ac: AM_GNU_GETTEXT is used, but not AM_GNU_GETTEXT_VERSION
#   cd "$E2FSPROGS_SRC"
#   autoreconf -i
#   cd ..
   rm -rf "$E2FSPROGS_SRC".obj
   mkdir -p "$E2FSPROGS_SRC".obj
   cd "$E2FSPROGS_SRC".obj
   CPPFLAGS="-I$TARGET_ROOT/include" \
   CONFIG_SITE=$CONFIG_SYSTEM/e2fsprogs-site.hurd-i386 \
   ../$E2FSPROGS_SRC/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --prefix="$TARGET_ROOT" \
      --enable-elf-shlibs \
      --disable-ubsan \
      --disable-addrsan \
      --disable-threadsan \
      --disable-e2initrd-helper \
      --disable-fsck \
      --disable-libblkid \
      --enable-libuuid  \
      --disable-uuidd \
      --enable-symlink-install \
      --disable-fuse2fs

   make -j$PROCS
   fakeroot make install
   fakeroot make install-libs
   cd ..
   rm -rf "$E2FSPROGS_SRC".obj
}

#Note: install-sh:
# -C install only if different (preserve the last data modification time)
#   LDFLAGS="-Wl,-verbose,-L$TARGET_ROOT/lib -lblkid -luuid -lz -lncursesw -ltinfow -ldl" \
#
    
install_hurd()
{
   print_info "Installing Hurd"
   cd "$HURD_SRC"
   autoreconf -i
   cd ..
   rm -rf "$HURD_SRC".obj
   mkdir -p "$HURD_SRC".obj
   cd "$HURD_SRC".obj
   CPPFLAGS="-I$TARGET_ROOT/include -fno-omit-frame-pointer" \
   LDFLAGS="-Wl,-verbose,-L$TARGET_ROOT/lib -Wl,-rpath-link,$TARGET_ROOT/lib" \
   ../$HURD_SRC/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --prefix="$TARGET_ROOT" \
      --without-parted \
      --enable-static-progs=ext2fs,iso9660fs,pci-arbiter \
      --with-ncursesw-include-dir="$TARGET_ROOT/include/ncursesw" \
      --with-libgcrypt-prefix="$TARGET_ROOT" \
      --with-libcrypt \
      --with-libz \
      --without-rump \
      --without-acpica \
      --without-libdaemon \
      --without-libbz2 \
      --enable-ncursesw \
      --disable-profile
   make -j$PROCS all || exit 1
   fakeroot make install || exit 1
   cd ..
   #   rm -rf "$HURD_SRC".obj
}

#NOTE: id and groups are from coreutils, su (and sulogin) from util-linux,
# login from hurd
#NOTE: config-site/shadow-site.hurd-i386 is needed!
install_shadow ()
{
   print_info "Installing shadow"
   cd "$SHADOW_SRC"
#   libtoolize: Consider adding '-I m4' to ACLOCAL_AMFLAGS in Makefile.am.
   autoreconf -i
   cd ..
   rm -rf "$SHADOW_SRC".obj
   mkdir "$SHADOW_SRC".obj
   cd "$SHADOW_SRC".obj
   CPPFLAGS="-I$TARGET_ROOT/include" \
   LDFLAGS="-Wl,-rpath-link,$SRC/$SHADOW_SRC.obj/libsubid/.libs -L$TARGET_ROOT/lib -lpam -lpam_misc" \
   CONFIG_SITE=$CONFIG_SYSTEM/shadow-site.hurd-i386 \
   ../$SHADOW_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --cache-file=config.cache \
     --enable-shared \
     --with-libpam \
     --without-su \
     --with-yescrypt \
     --enable-subordinate-ids
   make -j$PROCS all
#   make V=1 -j$PROCS all
   fakeroot make install
   cd ..
   rm -rf "$SHADOW_SRC".obj
}

install_grub()
{
   print_info "Installing grub"
   cd "$GRUB_SRC"
   autoreconf -ivf
   cd ..
   rm -rf "$GRUB_SRC".obj
   mkdir -p "$GRUB_SRC".obj
   cd "$GRUB_SRC".obj
   ../$GRUB_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --disable-werror \
     --enable-grub-mkfont=no \
     --with-bootdir=$TARGET_ROOT/boot \
     --with-grubdir=$TARGET_ROOT/boot/grub
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$GRUB_SRC".obj
}

install_gmp()
{
   print_info "Installing gmp"
   cd "$GMP_SRC"
   autoreconf -ivf
   cd ..
   rm -rf "$GMP_SRC".obj
   mkdir -p "$GMP_SRC".obj
   cd "$GMP_SRC".obj
   ../"$GMP_SRC"/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT"
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$GMP_SRC".obj
}

install_mpfr()
{
   print_info "Installing mpfr"
   cd "$MPFR_SRC"
   autoreconf -i
   cd ..
   rm -rf "$MPFR_SRC".obj
   mkdir -p "$MPFR_SRC".obj
   cd "$MPFR_SRC".obj
   ../$MPFR_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --with-gmp-include="$TARGET_ROOT"/include \
     --with-gmp-lib="$TARGET_ROOT"/lib
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$MPFR_SRC".obj
}

install_mpc()
{
   print_info "Installing mpc"
   cd "$MPC_SRC"
   autoreconf -i
   cd ..
   rm -rf "$MPC_SRC".obj
   mkdir -p "$MPC_SRC".obj
   cd "$MPC_SRC".obj
   ../"$MPC_SRC"/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --with-gmp-include="$TARGET_ROOT"/include \
     --with-gmp-lib="$TARGET_ROOT"/lib
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$MPC_SRC".obj
}

# configure.ac:34: error: Please use exactly Autoconf 2.69 instead of 2.71.
# apt-get install autoconf2.69
install_gcc()
{
   print_info "Installing gcc"
   #Note: This does not work unless autoconf2.69 is installed.
   #cd "$GCC_SRC"
   #autoreconf -i
   #cd ..
   rm -rf "$GCC_SRC".obj
   mkdir -p "$GCC_SRC".obj
   cd "$GCC_SRC".obj
    ../$GCC_SRC/configure \
     LDFLAGS="-L$TARGET_ROOT/lib" \
     --build="$HOST" \
     --host="$TARGET" \
     --target="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --with-gmp="$TARGET_ROOT" \
     --with-mpfr="$TARGET_ROOT" \
     --with-mpc="$TARGET_ROOT" \
     --enable-linker-build-id \
     --enable-lto \
     --enable-default-pie \
     --disable-multilib \
     --disable-nls \
     --disable-bootstrap \
     --enable-languages=c,c++ \
     --disable-libstdcxx-pch \
     --with-system-zlib \
     --disable-libcilkrts \
     --disable-libssp
   make -j$PROCS V=1 all
   fakeroot make install
   cd ..
   rm -rf "$GCC_SRC".obj
}

install_glibc ()
{
   print_info "Installing glibc"
   cd "$GLIBC_SRC"
   autoreconf -i
   cd ..
   rm -rf "$GLIBC_SRC".obj
   mkdir -p "$GLIBC_SRC".obj
   cd "$GLIBC_SRC".obj
   rm -f config.cache
   CPPFLAGS="-fno-omit-frame-pointer" \
   ../"$GLIBC_SRC"/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --without-selinux \
     --enable-obsolete-rpc \
     --disable-profile \
     --disable-nscd \
     --enable-stackguard-randomization \
     --enable-stack-protector=strong \
     --enable-add-ons=libidn,"" \
     --enable-pt_chown \
     --disable-mathvec \
     --disable-werror
   make -j$PROCS
   fakeroot make install
   cd ..
#   rm -rf "$GLIBC_SRC".obj
}

#   LDFLAGS="-L$TARGET_ROOT/lib -lncursesw -ltinfow -lncurses -ltinfo"
install_nano()
{
   print_info "Installing nano"
#   cd "$NANO_SRC"
#   autoreconf -i
#   cd ..
   rm -rf "$NANO_SRC".obj
   mkdir -p "$NANO_SRC".obj
   cd "$NANO_SRC".obj
   CPPFLAGS="-I$TARGET_ROOT/include -I$TARGET_ROOT/include/ncursesw" \
   LDFLAGS="-L$TARGET_ROOT/lib -ltinfow" \
   CONFIG_SITE=$CONFIG_SYSTEM/nano-site.hurd-i386 \
   ../$NANO_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --enable-utf8=yes
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$NANO_SRC".obj
}

install_less()
{
   print_info "Installing less"
#   cd "$LESS_SRC"
#   autoreconf -i
#   cd ..
   rm -rf "$LESS_SRC".obj
   mkdir -p "$LESS_SRC".obj
   cd "$LESS_SRC".obj
   CPPFLAGS="-I$TARGET_ROOT/include -I$TARGET_ROOT/include/ncursesw" \
   LDFLAGS="-L$TARGET_ROOT/lib -ltinfow" \
   ../$LESS_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --with-editor=nano
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$LESS_SRC".obj
}

#   CPPFLAGS="-I$TARGET_ROOT/include -I$TARGET_ROOT/include/ncursesw" \
#   LDFLAGS="-L$TARGET_ROOT/lib -ltinfow" \
	   
install_grep()
{
   print_info "Installing grep"
   cd "$GREP_SRC"
   autoreconf -i
   cd ..
   rm -rf "$GREP_SRC".obj
   mkdir -p "$GREP_SRC".obj
   cd "$GREP_SRC".obj
   ../$GREP_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --disable-nls
   make -j$PROCS
   fakeroot make install
   cd ..
   #rm -rf "$GREP_SRC".obj
}

install_vim () {
  print_info "Installing vim"
  cd vim"$VIM_BASE_VERSION"
  cat > src/auto/config.cache << "EOF"
  vim_cv_getcwd_broken=no
  vim_cv_memmove_handles_overlap=yes
  vim_cv_stat_ignores_slash=no
  vim_cv_terminfo=yes
  vim_cv_toupper_broken=no
  vim_cv_tty_group=world
EOF
  echo "#define SYS_VIMRC_FILE \"${TARGET_ROOT}/etc/vimrc\"" >> src/feature.h
  ./configure --build=${HOST} \
     --host=${TARGET} \
    --prefix=${TARGET_ROOT} \
    --enable-gui=no \
    --disable-gtktest \
    --disable-xim \
    --disable-gpm \
    --without-x \
    --disable-netbeans \
    --with-tlib=ncurses
  make
  make install
  ln -sfv vim $TARGET_ROOT/bin/vi
  cd ..
  cat > $TARGET_ROOT/etc/vimrc << "EOF"
set nocompatible
set backspace=2
set expandtab
set ts=2
set ruler
syntax on
EOF
}
