#!/bin/bash

# Copyright (C) 2019-2023 Svante Signell

# Written by Svante Signell  <svante.signell@gmail.com>

# This file is part of hurd-cross.

# Hurd-cross is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 3 or (at your
# option) any later version.

# Hurd-cross is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the hurd-cross.  If not, see <http://www.gnu.org/licenses/>.

. ./package-versions.sh
. ./environment-variables.sh

echo ""
echo "File: create-image.sh"

LOOP=/dev/loop0
MAPPER1=/dev/mapper/loop0p1
MAPPER2=/dev/mapper/loop0p2
MAPPER3=/dev/mapper/loop0p3
IMAGE_SIZE=8000MiB

create_image () {
    print_info "create_image(): Creating disk image $TARGET_IMAGE..."
    fallocate -l $IMAGE_SIZE $TARGET_IMAGE
    losetup $LOOP $TARGET_IMAGE
    parted -a optimal -s $LOOP mklabel msdos
    parted -a optimal -s $LOOP -- mkpart primary ext2 1049k 4000MB
    parted -a optimal -s $LOOP -- mkpart primary ext2 4000MB 7800MB
    parted -a optimal -s $LOOP -- mkpart primary linux-swap 7800MB -1
    parted -s $LOOP -- set 1 boot on
    parted -s $LOOP -- set 3 swap on
    kpartx -av $LOOP
    print_info "create_image(): Done creating partitions on $TARGET_IMAGE..."
    sleep 2
    print_info "create_image(): Creating ext2fs on /"
    mkfs.ext2 -o hurd -m 1 -v $MAPPER1
    print_info "create_image(): Creating ext2fs on /home"
    mkfs.ext2 -o hurd -m 1 -v $MAPPER2
    print_info "create_image(): Creating swap"
    mkswap          --verbose $MAPPER3
}

mount_image () {
    print_info "mount_image(): Mounting disk image mnt1, mnt2"
    mkdir -p mnt1
    mount -t ext2 $MAPPER1 mnt1
    mkdir -p mnt2
    mount -t ext2 $MAPPER2 mnt2
}

# Additional stuff:
# mkdir -p files/etc/pam.d
# cp /etc/pam.d/common-auth files/etc/pam.d/common-auth
# FIXME: Create pam-auth-update in Linux-PAM
# file /usr/sbin/pam-auth-update
# /usr/sbin/pam-auth-update: Perl script text executable
# FIXME: Find out how and create /etc/skel files
# Note: To have an empty root password: remove second entry (*) in files/etc/shadow
# (same for (!) in $USER)

copy_files () {
    print_info "copy_files(): Copying system into ./mnt1"
    cp -pR /tools/* mnt1/
    mkdir -p mnt1/home
    mkdir -p mnt1/proc
    mkdir -p mnt1/var/run
    mkdir -p mnt1/var/log
    mkdir -p mnt1/var/spool/mail
    cp -pR files/etc mnt1
    mkdir -p mnt1/libexec
    cp -p files/libexec/{rc,runsystem,runsystem.hurd} mnt1/libexec
    cp -p files/sbin/shadowconfig mnt1/sbin
    cp -p files/bin/rgrep mnt1/bin
    mkdir -p mnt1/boot/grub
    cp -p files/boot/grub.cfg mnt1/boot/grub
    cp -p files/boot/servers.boot mnt1/boot
    mkdir -p mnt1/servers/socket
    touch mnt1/servers/{exec,crash-kill,default-pager,password,socket,startup,proc,procfs,auth,symlink}
    mkdir mnt1/root
    cp -pR files/root mnt1
    mkdir mnt1/tmp
    chmod 01777 mnt1/tmp
    (cd mnt1/bin; ln -sf /bin/bash ./sh)
    # Crucial link?
    (cd mnt1/lib; ln -sf ld.so.1 ld.so)
    (cd mnt1; ln -sf . ./tools)
    (cd mnt1; ln -sf . ./usr)
    # Run the first_boot code of libexec/runsystem
    touch mnt1/first_boot
    # Create a motd message.
    echo "Welcome to the Hurd, v$TARGET_VERSION!" > mnt1/etc/motd
    echo "Cross-compiled from $HOST to $TARGET on `date`" >> mnt1/etc/motd
    chown -R root:root mnt1
    # Restore setuid flag
    chmod u+s mnt1/bin/login
    chmod u+s mnt1/bin/passwd
    chmod u+s mnt1/bin/chfn
    chmod u+s mnt1/bin/chsh
    chmod u+s mnt1/bin/gpasswd
    chmod u+s mnt1/bin/newgrp
    chmod u+s mnt1/bin/w
    chmod u+s mnt1/bin/ids
    chmod u+s mnt1/bin/su
    chmod u+s mnt1/bin/ps
    (cd mnt1/etc; ln -sf UTC localtime)
}

install_grub () {
   print_info "install_grub(): Installing the GRUB on $TARGET_IMAGE..."
   grub-install --target=i386-pc --boot-directory=$PWD/mnt1/boot $LOOP
}

umount_image () {
    print_info "umount_image(): Unmounting mnt1, mnt2..."
    umount mnt1
    umount mnt2
    kpartx -d $LOOP
    losetup -d $LOOP
    rmdir mnt1
    rmdir mnt2
}

#umount mount >/dev/null 2>&1
#kpartx -d $LOOP >/dev/null 2>&1
#losetup -d $LOOP >/dev/null 2>&1
umount mnt1
umount mnt2
kpartx -d $LOOP
losetup -d $LOOP
rm -f $TARGET_IMAGE
create_image
mount_image
copy_files
install_grub
umount_image
chown $USER:$USER $TARGET_IMAGE
mv $TARGET_IMAGE $SYSTEM
print_info "Disk image available on $SYSTEM/$TARGET_IMAGE"
print_info "Run 'qemu-system-x86_64 $TARGET_IMAGE' to enjoy the Hurd!"
exit 0
